import java.util.*;
import java.util.Scanner;
import java.io.*;


public class TestingShell
{
	// Results will be appended to file each time program is run.
	static String RESULTS_FILE_NAME = "ExperimentResults.txt";

  	public static void main(String[] args) throws Exception 
	{
 		patternedValues();
 		randomValues();
	}

	public static void patternedValues()
	{
		// Time how long it takes to build a red-black tree with 100,000 patterned values.
		// Do this "numRep" (10) times and compute the average. Then repeat for 200,000; 300,000; ...
		// up to "end" (1,000,000).
      RedBlackTree<Integer> rbt;
		long bTime, eTime, tTime=0, ttTime = 0;
		double avgTime;
					
      int begin =  100001;
		int end =   1000001;
		int incr =   100000;
      int GAP  =      307;
		int numReps =    10;
		
		for( int numVals=begin; numVals<=end; numVals+=incr )
		{
			ttTime = 0;
			for( int rep=1; rep<=10; rep++ )
			{
				tTime = 0;
				rbt = new RedBlackTree<Integer>();
				
 	   	   for( int val = GAP; val != 0; val = ( val + GAP ) % numVals )
				{
					bTime = System.nanoTime();
					
					rbt.insert( val );
					
					eTime = System.nanoTime();		
					tTime += (eTime-bTime);
				}
				ttTime += tTime;
			}
			avgTime = ttTime / numReps;
			displayRunTime( "Red-Black-Patterned", (numVals-1), avgTime );	
		}

		// Time how long it takes to build a binary search tree with 100,000 patterned values.
		// Do this "numRep" (10) times and compute the average. Then repeat for 200,000; 300,000; ...
		// up to "end" (1,000,000).
      BinarySearchTree<Integer> bst;
		
		for( int numVals=begin; numVals<=end; numVals+=incr )
		{
			ttTime = 0;
			for( int rep=1; rep<=10; rep++ )
			{
				tTime = 0;
				bst = new BinarySearchTree<Integer>();
				
 	   	   for( int val = GAP; val != 0; val = ( val + GAP ) % numVals )
				{
					bTime = System.nanoTime();
					
					bst.insert( val );
					
					eTime = System.nanoTime();		
					tTime += (eTime-bTime);
				}
				ttTime += tTime;
			}
			avgTime = ttTime / numReps;
			displayRunTime( "BinarySearchTree-Patterned", (numVals-1), avgTime );	
		}
		
	}

	public static void randomValues()
	{
		// Time how long it takes to build a red-black tree with 100,000 random values.
		// Do this "numRep" (10) times and compute the average. Then repeat for 200,000; 300,000; ...
		// up to "end" (1,000,000).
      RedBlackTree<Integer> rbt;
		long bTime, eTime, tTime=0, ttTime = 0;
		double avgTime;
					
      int begin =  100000;
		int end =   1000000;
		int incr =   100000;
		int numReps =    10;
		
		for( int numVals=begin; numVals<=end; numVals+=incr )
		{
			ttTime = 0;
			for( int rep=1; rep<=10; rep++ )
			{
				tTime = 0;
				rbt = new RedBlackTree<Integer>();
				
				int val = 0;
				while ( val <= numVals )
				{
					int x = randInteger(0,10000000);
					
					if( rbt.find( x ) == null )
					{
						val++;
						bTime = System.nanoTime();
					
						rbt.insert( x );
						
						eTime = System.nanoTime();		
						tTime += (eTime-bTime);
					}
				}
				ttTime += tTime;
			}
			avgTime = ttTime / numReps;
			displayRunTime( "Red-Black-Random", numVals, avgTime );	
		}

		// Time how long it takes to build a binary search tree with 100,000 random values.
		// Do this "numRep" (10) times and compute the average. Then repeat for 200,000; 300,000; ...
		// up to "end" (1,000,000).
      BinarySearchTree<Integer> bst;
		
		for( int numVals=begin; numVals<=end; numVals+=incr )
		{
			ttTime = 0;
			for( int rep=1; rep<=10; rep++ )
			{
				tTime = 0;
				bst = new BinarySearchTree<Integer>();
				
				int val = 0;
				while ( val <= numVals )
				{
					int x = randInteger(0,10000000);
					
					if( bst.find( x ) == null )
					{
						val++;
						bTime = System.nanoTime();
					
						bst.insert( x );
						
						eTime = System.nanoTime();		
						tTime += (eTime-bTime);
					}
				}
				ttTime += tTime;
			}
			avgTime = ttTime / numReps;
			displayRunTime( "BinarySearchTree-Random", numVals, avgTime );	
		}
		
	}

	
//-----------------------------------------------------------------------
// Helper Methods
//-----------------------------------------------------------------------

	private static double randDouble(double a, double b)
	{
		return a + (b*Math.random());
	}

	private static int randInteger(double a, double b)
	{
		return (int)(a + (b*Math.random()) );
	}
	
	private static void displayRunTime( String msg, long vals, double tTime )
	{
		double runTimeSeconds = tTime/1000000000.0; 

		String output = String.format( "%s %d %f", msg, vals, runTimeSeconds );
				
		// Write output string to console.
		System.out.println( output );
	
		// Write output string to text file.
		File outFile = new File( RESULTS_FILE_NAME );
		try
		{
			FileWriter fw = new FileWriter( outFile, true );
 			PrintWriter writer = new PrintWriter( fw );
			writer.println( output );
			writer.close();
		}
		catch(IOException e )
		{
		}
	}
}

